@@ -57,7 +57,6 @@ const trx_t*		trx_roll_crash_recv_trx;
 
 bool trx_t::rollback_finish() noexcept
 {
-  mod_tables.clear();
   apply_online_log= false;
   if (UNIV_LIKELY(error_state == DB_SUCCESS))
   {
@@ -82,8 +81,7 @@ bool trx_t::rollback_finish() noexcept
     ut_free(undo);
     undo= nullptr;
   }
-  commit_low();
-  return commit_cleanup();
+  return commit();
 }
 
 dberr_t trx_t::rollback_low(const undo_no_t *savept) noexcept
@@ -139,9 +137,12 @@ dberr_t trx_t::rollback_low(const undo_no_t *savept) noexcept
       trx_mod_tables_t::iterator j= i++;
       ut_ad(j->second.valid());
       if (j->second.rollback(limit))
+      {
+        j->second.clear_bulk_buffer();
         mod_tables.erase(j);
+      }
       else if (!apply_online_log)
-        apply_online_log= j->first->is_active_ddl();
+        apply_online_log= j->first->is_native_online_ddl();
     }
     MONITOR_INC(MONITOR_TRX_ROLLBACK_SAVEPOINT);
   }
@@ -231,10 +232,10 @@ dberr_t trx_rollback_for_mysql(trx_t* trx)
 			the actions already having been rolled back. */
 			ut_ad(trx->rsegs.m_redo.undo->rseg
 			      == trx->rsegs.m_redo.rseg);
-			mtr_t		mtr;
+			mtr_t mtr{trx};
 			mtr.start();
 			if (trx_undo_t* undo = trx->rsegs.m_redo.undo) {
-				trx_undo_set_state_at_prepare(trx, undo, true,
+				trx_undo_set_state_at_prepare(undo, true,
 							      &mtr);
 			}
 			/* Write the redo log for the XA ROLLBACK
